package tcbnet.control;

import tcbnet.io.ObjectFile;
import tcbnet.ui.UIConfiguration;

import java.io.*;
import java.util.*;

/**
 * Processa, armazena, salva e carrega parametros de configuracao do programa.
 *
 * @author $Author: dseki $
 * @version $Revision: 1.24 $
 */

public class Configuration
    implements Serializable,
	       ConfigurationNode,
	       ConfigurationUI
{
    private static String CONFIG_FILE = "tcbnet.cfg";

    private int p2pMaxConnections;
    private int p2pMaxSeenNodes;
    private short p2pTimeToLive;

    private String treecastDefaultDirectory;
    private int treecastMaxConnections;
    private long treecastTimeout;

    private FilterRoot filterTree;
    private UIConfiguration uiConfiguration;

    /**
     * Construtor. Define os valores padrao para cada parametro.
     */
    public Configuration()
    {
	this.p2pMaxConnections = 4;
	this.p2pMaxSeenNodes = 20;
	this.p2pTimeToLive = 5;

	this.treecastDefaultDirectory = "";
	this.treecastMaxConnections = 4;
	this.treecastTimeout = 60;

	this.filterTree = new FilterRoot();
	this.uiConfiguration = new UIConfiguration();
    }

    /**
     * Salva este objeto de configuracao no arquivo de configuracao padrao.
     *
     * @throws IOException quando nao for possivel salvar o objeto de config
     */
    public synchronized void save()
	throws IOException
    {
	ObjectFile.saveObject(CONFIG_FILE, this);
    }

    /**
     * Carrega um objeto de configuracao do arquivo de configuracao padrao,
     * se for possivel. Senao cria um novo objeto com as configuracoes padroes.
     */
    public static synchronized Configuration load()
    {
	try {
	    return (Configuration) ObjectFile.loadObject(CONFIG_FILE);
	}
	catch (Exception e) {
	    return new Configuration();
	}
    }

    /**
     * Le do objeto de configuracoes o numero maximo de conexoes da
     * Peer-To-Peer Headers Network.
     *
     * @return numero maximo de conexoes da P2PHN
     */
    public synchronized int getP2PMaxConnections()
    {
	return this.p2pMaxConnections;
    }

    /**
     * Salva no objeto de configuracoes o numero maximo de conexoes da
     * Peer-To-Peer Headers Netword.
     *
     * @param p2pMaxConnections numero maximo de conexoes da P2PHN
     * @throws IllegalArgumentException quando o numero for menor do que 1
     */
    public synchronized void setP2PMaxConnections(int p2pMaxConnections)
	throws IllegalArgumentException
    {
	if (p2pMaxConnections < 1)
	    throw new IllegalArgumentException();
	this.p2pMaxConnections = p2pMaxConnections;
    }

    /**
     * Le do objeto de configuracoes o numero maximo de nodes a serem
     * armazenados visando novas conexoes na rede P2PHN.
     *
     * @return numero maximo de nodes conhecidos para guardar
     */
    public synchronized int getP2PMaxSeenNodes()
    {
	return this.p2pMaxSeenNodes;
    }

    /**
     * Salva no objeto de configuracoes o numero maximo de nodes a serem
     * armazenados visando novas conexoes na rede P2PHN.
     *
     * @param p2pMaxSeenNodes numero maximo de nodes a serem armazenados
     * @throws IllegalArgumentException quando o numero for menor do que 0
     */
    public synchronized void setP2PMaxSeenNodes(int p2pMaxSeenNodes)
	throws IllegalArgumentException
    {
	if (p2pMaxSeenNodes < 0)
	    throw new IllegalArgumentException();
	this.p2pMaxSeenNodes = p2pMaxSeenNodes;
    }

    /**
     * Le do objeto de configuracoes o Time-To-Live ("Tempo de Vida" ou nivel
     * de recursao) de um cabecalho (Header) na P2PHN.
     *
     * @return tempo de vida de um Header
     */
    public synchronized short getP2PTimeToLive()
    {
	return this.p2pTimeToLive;
    }

    /**
     * Salva no objeto de configuracoes o Time-To-Live de um cabecalho na
     * P2PHN.
     *
     * @param p2pTimeToLive tempo de vida do cabecalho na P2PHN
     * @throws IllegalArgumentException quando o tempo for menor do que 1
     */
    public synchronized void setP2PTimeToLive(short p2pTimeToLive)
	throws IllegalArgumentException
    {
	if (p2pTimeToLive < 1)
	    throw new IllegalArgumentException();
	this.p2pTimeToLive = p2pTimeToLive;
    }

    /**
     * Le do objeto de configuracao o diretorio padrao onde serao armazenados
     * os novos arquivos capturados pelo programa.
     *
     * @return string com o diretorio padrao para salvar os arquivos
     */
    public synchronized String getTreecastDefaultDirectory()
    {
	return this.treecastDefaultDirectory;
    }

    /**
     * Salva no objeto de configuracao o diretorio padrao onde deverao ser
     * armazenados os novos arquivos capturados pelo programa.
     *
     * @param treecastDefaultDirectory local padrao para salvar arquivos
     */
    public synchronized
	void setTreecastDefaultDirectory(String treecastDefaultDirectory)
    {
	this.treecastDefaultDirectory = treecastDefaultDirectory;
    }

    /**
     * Le do objeto de configuraco o numero maximo de conexoes da Treecast
     * Network (TN).
     *
     * @return numero maximo de conexoes da TN
     */
    public synchronized int getTreecastMaxConnections()
    {
	return this.treecastMaxConnections;
    }

    /**
     * Salva no objeto de configuracao o numero maximo de conexoes da TN.
     *
     * @param treecastMaxConnections numero maximo de conexoes da TN
     */
    public synchronized
	void setTreecastMaxConnections(int treecastMaxConnections)
	throws IllegalArgumentException
    {
	if (treecastMaxConnections < 1)
	    throw new IllegalArgumentException();
	this.treecastMaxConnections = treecastMaxConnections;
    }

    /**
     * Le do objeto de configuracao o tempo em segundos de espera antes de
     * parar um Push, ou seja, evitar que novas transmissoes sejam criadas para
     * ele.
     *
     * @return tempo em segundos de espera antes de parar push
     */
    public synchronized long getTreecastTimeout()
    {
	return this.treecastTimeout;
    }

    /**
     * Salva no objeto de configuracao o tempo em segundos de espera antes de
     * parar um Push, ou seja, evitar que novas transmissoes sejam criadas para
     * ele. Exige que o tempo seja pelo menos 60 segundos.
     *
     * @param treecastTimeout tempo em segundos de espera antes de parar push
     * @throws IllegalArgumentException quando o tempo eh menor do que 60
     */
    public synchronized void setTreecastTimeout(long treecastTimeout)
	throws IllegalArgumentException
    {
	if (treecastTimeout < 60)
	    throw new IllegalArgumentException();
	this.treecastTimeout = treecastTimeout;
    }

    /**
     * Le a arvore de filtros do arquivo de configuracoes.
     *
     * @return arvore de filtro
     */
    public synchronized FilterRoot getFilterTree()
    {
	return this.filterTree;
    }

    /**
     * Salva nas configuracoes uma arvore de filtros.
     *
     * @param filterTree arvore a ser salva
     */
    public synchronized void setFilterTree(FilterRoot filterTree)
    {
	this.filterTree = filterTree;
    }

    /**
     * Le deste objeto de configuracao um objeto de configuracao de interface
     * com o usuario (UI).
     *
     * @param uiConfiguration objeto de configuracao de interface
     */
    public synchronized UIConfiguration getUIConfiguration()
    {
	return this.uiConfiguration;
    }

    /**
     * Salva neste objeto de configuracao um objeto de configuracao de
     * interface com o usuario (UI).
     *
     * @param uiConfiguration objeto de configuracao de interface
     */
    public synchronized
	void setUIConfiguration(UIConfiguration uiConfiguration)
    {
	this.uiConfiguration = uiConfiguration;
    }
}
