package tcbnet.control;

import tcbnet.corba.Header;

/**
 * Filtro de tamanho de arquivo. O tamanho do arquivo deve estar no intervalo
 * especificado, em bytes.
 *
 * @author $Author: dseki $
 * @version $Revision: 1.7 $
 */

public class FilterFilesize
    extends FilterNode
{
    private long maxSize;
    private long minSize;

    /**
     * Construtor da classe FilterFilesize. Cria um filtro de tamanho de arqui-
     * vo com o tamanho minimo como 0 e o tamanho maximo como o tamanho maximo
     * de long.
     */
    public FilterFilesize()
    {
	this(0, Long.MAX_VALUE);
    }

    /**
     * Construtor da classe FilterFilesize. Cria um filtro de tamanho de arqui-
     * vo com o tamanho minimo e o tamanho maximo com os valores espeficicados.
     *
     * @param minSize tamanho minimo do arquivo
     * @param maxSize tamanho maximo do arquivo
     */
    public FilterFilesize(long minSize, long maxSize)
    {
	super();
	this.setMinSize(minSize);
	this.setMaxSize(maxSize);
    }

    /**
     * Retorna o tamanho minimo do arquivo, em bytes, para ele passar pelo
     * filtro.
     *
     * @return tamanho minimo do arquivo
     */
    public synchronized long getMinSize()
    {
	return this.minSize;
    }

    /**
     * Define o tamanho minimo do arquivo, em bytes, para ele passar pelo
     * filtro.
     *
     * @param minSize tamanho minimo do arquivo
     * @throws IllegalArgumentException
     */
    public synchronized void setMinSize(long minSize)
	throws IllegalArgumentException
    {
	if (minSize < 0)
	    throw new IllegalArgumentException();
	this.minSize = minSize;
    }

    /**
     * Retorna o tamanho maximo do arquivo, em bytes, para ele passar pelo
     * filtro.
     *
     * @return tamanho maximo do arquivo
     */
    public synchronized long getMaxSize()
    {
	return this.maxSize;
    }

    /**
     * Define o tamanho maximo do arquivo, em bytes, para ele passar pelo
     * filtro.
     *
     * @param maxSize tamanho maximo do arquivo
     * @throws IllegalArgumentException
     */
    public synchronized void setMaxSize(long maxSize)
	throws IllegalArgumentException
    {
	if (maxSize < 0)
	    throw new IllegalArgumentException();
	this.maxSize = maxSize;
    }

    /**
     * Dado um header, decide se ele passa no filtro ou nao.
     * Passa pelo filtro se o tamanho do arquivo descrito no header for maior
     * ou igual o tamanho minimo e, ao mesmo tempo, menor ou igual ao tamanho
     * maximo.
     *
     * @param h header a ser avaliado
     * @return se o arquivo do cabecalho passa pelo filtro ou nao
     */
    public synchronized boolean passFilter(Header h)
    {
	if (h == null)
	    return false;
	if (this.minSize <= h.fileSize && h.fileSize <= this.maxSize)
	    return true;
	return false;
    }

    /**
     * Retorna uma representacao do filtro no formato de texto.
     *
     * @return representacao no formato texto do filtro
     */
    public synchronized String toString()
    {
	return
	    "Filesize ["
	    + Long.toString(this.minSize / 1024)
	    + ","
	    + Long.toString(this.maxSize / 1024)
	    + "]";
    }

    public Object clone() {
	Object copy = null;
        copy = super.clone();
        // Precisa?
	((FilterFilesize)copy).setMinSize(minSize);
	((FilterFilesize)copy).setMaxSize(maxSize);
	return copy;

    }

}
