package tcbnet.corba;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Collection;
import tcbnet.treecast.*;
import tcbnet.io.*;
import tcbnet.control.*;

/**
 * <p>Title: TCBnet</p>
 * <p>Description: p2p content push</p>
 * <p>Copyright: TCB inc Copyright (c) 2002</p>
 * <p>Company: TCB inc</p>
 * @author $Author: dseki $
 * @version $Revision: 1.28 $
 */

public class NodeImpl extends NodePOA {
  String ior;
  TcbnetOrb orb;
  P2PNodeList p2pNodeList;
  P2PKeepAlive p2pKeepAlive;
  TreecastControl treecastControl;

  /**
   * Construtor para um Node
   * @param cfg o P2PRelay associado
   */
  public NodeImpl(ConfigurationNode cfg) {
    ior = null;
    orb = TcbnetOrb.getInstance();

    treecastControl =
      new TreecastControl(orb.getNode(this),
          cfg.getP2PTimeToLive(), cfg.getTreecastTimeout(),
          cfg.getTreecastDefaultDirectory(), cfg.getTreecastMaxConnections());

    p2pNodeList = new P2PNodeList(treecastControl,
				  cfg.getFilterTree(),
				  cfg.getP2PMaxConnections(),
				  cfg.getP2PMaxSeenNodes());

    p2pKeepAlive =
      new P2PKeepAlive(orb.getNode(this), p2pNodeList,
        cfg.getP2PMaxConnections());

  }

  // ------------------- CORBA IDL ------------------------//

  /**
   * Anuncia a existencia de um novo n, e este  adicionado a lista de ns
   * conhecidos.
   * @param n o n que se anuncia
   * @throws NoConnectionAvailableException erro caso haja um erro na conexo
   */
  public void announce(Node n)
    throws NoConnectionAvailableException {
      System.out.println ("announce : NodeImpl");
      p2pNodeList.announce(n);
  }

  /**
   * Recebe um novo header enviado por outro n
   * @param h o header
   */
  public void sendHeader(Header h) {
      System.out.println("enviando header " + this);
      p2pNodeList.sendHeader(orb.getNode(this), h);
  }

  /**
   * Devolve a Stream que possui o checksum correspondente
   * @param checkSum o checksum que identifica o arquivo
   * @return a stream
   * @throws NoStreamAvailableException
   * @throws NotAvailableException
   */
  public Stream getStream(Node n, String checkSum)
    throws NoStreamAvailableException, NotAvailableException {
    return treecastControl.getStream(n, checkSum);
  }

  // ------------------- CORBA IDL ------------------------ //

  /**
   * Devolve a ior correspondente ao objeto Node
   * @return a String com o ior
   */
  public String getIor() {
    if (this.ior == null) {
      TcbnetOrb orb = TcbnetOrb.getInstance();
      ior = orb.getIor(this);
    }
    return ior;
  }

  // -------------------- EXTERNOS ----------------------- //

    /**
     * Tenta se conectar a um n na rede P2P
     * @param node o n em que deve-se tentar conectar
     */
    public void tryConnect(Node node) {
      this.p2pKeepAlive.add(node);
    }

    /**
     * Tenta se conectar a alguns nos na rede P2P
     * @param nodes lista de nos que devem ser conectados a this
     */
    public void tryConnect(Node[] nodes) {
	for (int i = 0; i < nodes.length; i++) this.tryConnect(nodes[i]);
    }

    /**
     * Realiza o push de um arquivo
     * @param fileName o nome do arquivo
     * @param desc a descrio do arquivo
     * @return Retorna um array de objetos com 2 posicoes onde o primeiro e o
     *         Header criado e o segundo, uma String com o path do arquivo.
     * @exception FileNotFoundException ao criar um header
     * @exception IOException ao criar um header
     */
    public Object[] prepareFile(String filename, String desc)
	throws FileNotFoundException, IOException {

	FileHeader fh = new FileHeaderImpl();
	Header h = fh.getHeaderFromFile(filename, desc);

	String path = "";
	String file = "";

	int i1 = filename.lastIndexOf("/");
	int i2 = filename.lastIndexOf("\\");

	if (i1 != 0) {
	    path = filename.substring(0, i1+1);
	    file = filename.substring(i1+1);
	    System.out.println("path $>" + path + "\t file: " + file);
	}
	else if (i2 != 0) {
	    path = filename.substring(0, i2+1);
	    file = filename.substring(i2+1);
	    System.out.println("path $>" + path + "\t file: " + file);
	} else {
	    path = "";
	    file = filename;
	    System.out.println("path $>" + path + "\t file: " + file);
	}

	h.fileName = file;

	Node local = orb.getNode(this); 
	h.node = local;
	h.visited[0] = local;
	Object[] objs = new Object[2];
	objs[0] = h;
	objs[1] = path;
	return objs;
    }

    /**
     * Realiza o push de um arquivo
     * @param fileName o nome do arquivo
     * @param desc a descrio do arquivo
     * @exception FileNotFoundException ao criar um header
     * @exception IOException ao criar um header
     */
    public Treecast pushHeader(Header h, String path) {
	return treecastControl.addTreecast(p2pNodeList, h, path);
    }

    /**
     * Retorna informacoes sobre downloads.
     *
     * @param i i-esimo download.
     * @param j j-esima informacao de download.
     * @return j-esima dentre "Filename", "Transfered", "Filesize", "Speed",
     *         "Source", "Description", "Progress" do i-esimo download.
     */
    public Object getDownloadInfo (int i, int j) {
	return treecastControl.getDownloadInfo (i, j);
    }

    /**
     * @return o numero de downloads ativos
     */
    public int getDownloadCount () {
	return treecastControl.getDownloadCount ();
    }

    /**
     * Atualiza as configuracoes.
     */
    public void update() {
	
    }

    /**
     * Assassina uma treecast (todas as transmissoes ja devem ter acabado e o
     * treecast deve estar parado).
     */
    public void kill(Treecast treecast) {
    }
}

