package tcbnet.treecast;

import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import tcbnet.control.*;
import tcbnet.corba.*;
import tcbnet.ui.*;
import tcbnet.util.*;

/**
 * Classe responsavel por lidar com os Nodes conectados ao Node local na rede
 * P2PHN. Assume-se por Node conectado, aquele Node que nao gerou excecao ao
 * chamar o metodo announce dele.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.40 $
 */
public class P2PNodeList {
    private int maxConnections;
    private int maxLastSeenNodes;
    private TreecastControl treecastControl;
    private Filter filter;
    private Map nodeHandlers;
    private List lastSeenNodes;

    /**
     * Construtor.
     *
     * @param tc TreecastControl para criacao de novos Treecasts.
     * @param f Filtro de Header a ser avaliado.
     * @param mc Numero maximo de Nodes conectados na rede P2PHN.
     * @param mlsn Numero maximo de Nodes vistos em Headers a ser amarzenados.
     */
    public P2PNodeList (TreecastControl tc, Filter f, int mc, int mlsn) {
	this.treecastControl = tc;
	this.filter = f;
	this.maxConnections = mc;
	this.maxLastSeenNodes = mlsn;
	this.nodeHandlers = new HashMap ();
	this.lastSeenNodes = new ArrayList ();
    }

    /**
     * Configura o numero maximo de Nodes conectados na rede P2PHN.
     *
     * @param mc Novo numero maximo de Nodes conectados na rede P2PHN.
     */
    public synchronized void setMaxConnections (int mc) {
	this.maxConnections = mc;
    }

    /**
     * Configura filtro a ser utilizado para processar Headers.
     *
     * @param f Novo filtro.
     */
    public synchronized void setFilter (Filter f) {
	this.filter = f;
    }

    /**
     * Configura o numero maximo de Nodes a serem armazenados.
     *
     * @param n Novo numero maximo de Nodes a serem armazenados.
     */
    public synchronized void setMaxLastSeenNodes (int n) {
	this.maxLastSeenNodes = n;
    }

    /**
     * Adiciona um Node a lista de Nodes adjacentes, caso haja conexoes
     * disponiveis.
     *
     * @param n Node a ser adicionado.
     * @throws NoConnectionAvailableException
     */
    public synchronized void announce (Node n)
	throws NoConnectionAvailableException {
	if (this.nodeHandlers.size () < this.maxConnections) {
	    if (!this.nodeHandlers.containsKey (n))
		this.add (n);
	    else
		return;
	}
	else
	    throw new NoConnectionAvailableException (this.getChildren ());
    }

    /**
     * Adiciona um Node na lista de Nodes adjacentes.
     *
     * @param n Node a ser adicionado.
     */
    public synchronized void add (Node n) {
	this.nodeHandlers.put (n, new NodeHandler (n));
    }
    
    /**
     * Retorna o numero de Nodes na lista de Nodes adjacentes.
     *
     * @return Numero de Nodes na lista de Nodes adjacentes.
     */
    public synchronized int size () {
	return this.nodeHandlers.size ();
    }
    
    /**
     * Retorna a lista com as referencias para os Nodes conectados ao Node
     * local na rede P2PHN.
     *
     * @return Lista com as referencias para os Nodes conectados ao Node local
     *         na rede P2PHN
     */
    private Node[] getChildren () {
	Object[] array = this.nodeHandlers.keySet ().toArray ();
	Node[] a = new Node[array.length];
	for (int i = 0; i < array.length; i++) a[i] = (Node) array[i];
	return a;
    }

    /**
     * Envia um Header para todos os Nodes conectados, exceto para o que o
     * mandou.
     *
     * @param n Node que enviou o Header.
     * @param h Header a ser enviado.
     */
    public synchronized void sendHeaderRaw (Node n, Header h) {
	System.out.println ("sendHeaderRaw : P2PNodeList");
	Iterator i = this.nodeHandlers.entrySet ().iterator ();

	while (i.hasNext ()) {
	    Map.Entry me = (Map.Entry) i.next ();
	    if (((Node) me.getKey ()).equals (n)) continue;
	    ((NodeHandler) me.getValue ()).sendHeader (h);
	}
    }

    /**
     * Recebe um Header, processa-o atraves dos filtros e o retransmite.
     *
     * @param ln O Node local.
     * @param h Header recebido da rede P2P.
     */
    public synchronized void sendHeader (Node ln, Header h) {
	System.out.println ("sendHeader : P2PNodeList");
	Node[] visited = new Node[h.visited.length + 1];
	int i;
	for (i = 0; i < h.visited.length; i++) {
	    if (h.visited[i].equals(ln)) {
		System.out.println ("ei! eu(" + ln+ ") jah to no visited:");
		System.out.println (h.visited);
		return;
	    }
	    visited[i] = h.visited[i];
	}
	visited[i] = ln;
	this.addNode (h.node);
	if (this.filter.passFilter (h)) {
	    System.out.println ("quero esse arquivo: " + h.fileName);
	    try { this.treecastControl.addTreecastReceiver (this, h); }
	    catch (FileNotFoundException fnfe) { }
	}
	else {
	    h.timeToLive--;
	    h.visited = visited;
	    this.sendHeaderRaw (null, h);
	}
    }

    /**
     * Retorna um Array com os ultimos Nodes vistos.
     *
     * @return Array com os ultimos Nodes vistos.
     */
    public synchronized Node[] getLastSeenNodes () {
	Object[] obj = lastSeenNodes.toArray ();
	Node[] nodes = new Node[obj.length];
	for (int i = 0; i < nodes.length; i++) {
	    nodes[i] = (Node) obj[i];
	}
	return nodes;
    }

    /**
     * Adiciona o Node n na lista de ultimos Nodes vistos se ele nao estiver
     * ja inserido. Caso a lista esteja cheia, remove o Node visto ha mais
     * tempo.
     *
     * @param n Node a ser inserido
     */
    private void addNode (Node n) {
	/** @todo pensar manter isso num vetor ordenado */
	if (! this.lastSeenNodes.contains (n)) {
	    if (this.lastSeenNodes.size () >= this.maxLastSeenNodes)
		this.lastSeenNodes.remove (0);
	    this.lastSeenNodes.add (n);
	}
    }
}

/**
 * Responsavel por cuidar da conexao de um Node.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.40 $
 */
class NodeHandler extends Thread {

    private Node node;
    private ProdConsMonitor headerBuffer;

    /**
     * Construtor.
     *
     * @param n Node que sera cuidado.
     */
    public NodeHandler (Node n) {
	this.node = n;
	this.headerBuffer = new ProdConsMonitor ();
	this.start ();
    }

    /**
     * Para cada Header contido na fila de envio, tenta enviar para o Node.
     * Caso a fila esteja vazia, dorme.
     */
    public void run () {
	Header h;
	while (true) {
	    h = (Header) this.headerBuffer.take ();
	    if (h != null) this.node.sendHeader (h);
	}
    }

    /**
     * Adiciona Header na Fila de envio do Node.
     *
     * @param h Header a ser enviado.
     */
    public synchronized void sendHeader (Header h) {
	headerBuffer.put (h);
    }

    /**
     * Retorna o Node associado.
     *
     * @return Node associado.
     */
    public synchronized Node getNode () {
	return this.node;
    }

}
