package tcbnet.ui;

import tcbnet.corba.Header;
import tcbnet.treecast.Treecast;

import java.io.*;

/**
 * Elemento/Item da fila de arquivos esperando o push.
 *
 * @author $Author: dseki $
 * @version $Revision: 1.14 $
 */

class PushItem
    implements Serializable
{
    private Header header;
    private String path;
    private String status;
    private Treecast treecast;
    private long startTime;
    private int transfersDone;

    /**
     * Construtor.
     *
     * @param header cabecalho de transmissao
     * @param path caminho completo ate o arquivo do cabecalho
     */
    PushItem(Header header, String path)
    {
	this.header = header;
	this.path = path;
	this.status = "Queued";
	this.transfersDone = 0;
    }

    /**
     * Devolve a descricao do arquivo.
     *
     * @return descricao do arquivo
     */
    synchronized String getDescription()
    {
	return this.header.fileDesc;
    }

    /**
     * Devolve o nome do arquivo na fila, incluindo localizacao relativa.
     *
     * @return nome do arquivo
     */
    synchronized String getFilename()
    {
	return this.header.fileName;
    }

    /**
     * Devolve o tamanho do arquivo do item, em bytes.
     *
     * @return tamanho do arquivo
     */
    synchronized long getFilesize()
    {
	return this.header.fileSize;
    }

    /**
     * Devolve o cabecalho do arquivo armazenado.
     *
     * @return cabecalho do arquivo
     */
    synchronized Header getHeader()
    {
	return this.header;
    }

    /**
     * Devolve o caminho completo ate o arquivo a ser enviado pelo item.
     *
     * @return caminho completo do arquivo
     */
    synchronized String getPath()
    {
	return this.path;
    }

    /**
     * Devolve o status do item.
     *
     * @return status
     */
    synchronized String getStatus()
    {
	return this.status;
    }

    /**
     * Devolve o tempo transcorrido de broadcast em milisegundos, ou seja,
     * o tempo limite de referencia menos o tempo transcorrido de broadcast.
     *
     * @param treecastTimeout tempo limite de referencia
     * @return tempo restante de broadcast
     */
    synchronized long getTimeLeft(long treecastTimeout)
    {
	if (this.status.equals("Pushing")) {
	    long now = System.currentTimeMillis();
	    long tmp = treecastTimeout - (now - this.startTime);
	    if (tmp >= 0) return tmp;
	}
	return 0;
    }

    /**
     * Devolve o numero de transferencias desse arquivo ja completadas.
     *
     * @return numero de transferencias completadas
     */
    synchronized int getTransfersDone()
    {
	if (this.status.equals("Queued") || this.status.equals("Done"))
	    return this.transfersDone;
	return this.treecast.getStreamDoneCount();
    }

    /**
     * Devolve o numero de transferencias em progresso desse arquivo.
     *
     * @return numero de transferencias em progresso
     */
    synchronized int getTransfersInProgress()
    {
	if (this.status.equals("Queued") || this.status.equals("Done"))
	    return 0;
	return this.treecast.getStreamCount();
    }

    /**
     * Retorna o treecast deste item.
     *
     * @return treecast do PushItem
     */
    synchronized Treecast getTreecast()
    {
	return this.treecast;
    }

    /**
     * Procedimento para alterar estado do item para o estado de realizando o
     * broadcast.
     *
     * @param treecast treecast gerado pelo envio do header
     */
    synchronized void startPush(Treecast treecast)
    {
	this.status = "Pushing";
	this.treecast = treecast;
	this.startTime = System.currentTimeMillis();
    }

    /**
     * Procedimento para alterar o estado do item para o estado apos o termino
     * do tempo limite do broadcast.
     */
    synchronized void stopPush()
    {
	this.treecast.stop();
	this.status = "Stopping";
    }

    /**
     * Procedimento para alterar o estado item para o estado apos o termino
     * do tempo limite e sem nenhum envio em andamento (acabou MESMO).
     *
     * @param transfersDone numero de transferencias realizadas
     */
    synchronized void buryPush(int transfersDone)
    {
	this.status = "Done";
	this.transfersDone = transfersDone;
	this.treecast = null;
    }

    /**
     * Atualiza os dados do Item de push. Na realidade, cria um novo PushItem
     * e substitui o antigo.
     *
     * @param header novo cabecalho de transmissao
     * @param path caminho completo ate o novo arquivo do cabecalho
     */
    synchronized void update(Header header, String path)
    {
	if (!this.status.equals("Queued"))
	    return;
	this.header = header;
	this.path = path;
    }
}
