package tcbnet.ui;

import tcbnet.corba.Header;
import tcbnet.treecast.Treecast;

import java.io.*;
import java.util.*;
import javax.swing.table.*;

/**
 * Fila de arquivos esperando o push, implementando um TableModel para poder
 * ser utilizada pela interface (e.g.: JTable).
 *
 * @author $Author: dseki $
 * @version $Revision: 1.18 $
 */

public class PushQueue
    extends AbstractTableModel
    implements Serializable
{
    private static int NCOLUMNS = 8;

    private ArrayList queue = null;
    private int queueActive = 0;

    private UIController uiController = null;

    /**
     * Construtor da classe PushQueue.
     */
    public PushQueue()
    {
	this.queue = new ArrayList();
	this.queueActive = 0;
    }

    /**
     * Retorna a superclasse mais especifica de todos os valores das celulas na
     * coluna. Este metodo e' utilizado por JTable para definir um renderizador
     * e um editor padroes para a coluna.
     *
     * <p>Origem : extends AbstractTableModel</p>
     *
     * @param columnIndex indice da coluna da tabela
     * @return a classe ancestral comum de todos os valores da coluna
     * @throws IndexOutOfBoundsException se a coluna for invalida
     */
    public synchronized Class getColumnClass(int columnIndex)
	throws IndexOutOfBoundsException
    {
	String s = new String();
	Integer i = new Integer(0);
	Long l = new Long(0);
	switch (columnIndex) {
	case  0 : return s.getClass();
	case  1 : return s.getClass();
	case  2 : return s.getClass();
	case  3 : return s.getClass();
	case  4 : return l.getClass();
	case  5 : return i.getClass();
	case  6 : return i.getClass();
	case  7 : return s.getClass();
	default : throw new IndexOutOfBoundsException();
	}
    }

    /**
     * Devolve o numero de colunas na tabela, que eh o numero de campos de um
     * PushItem.
     *
     * <p>Origem : extends AbstractTableModel</p>
     *
     * @return numero de colunas da tabela
     */
    public synchronized int getColumnCount()
    {
	return this.NCOLUMNS;
    }

    /**
     * Devolve o nome da coluna na posicao passada como parametro. Utilizado
     * para inicializar o nome do cabecalho da coluna da tabela. Nota: este
     * nome nao precisa ser unico - duas colunas da tabela podem ter o mesmo
     * nome.
     *
     * <p>Origem : extends AbstractTableModel</p>
     *
     * @param columnIndex indice da coluna da tabela
     * @return o nome da coluna
     * @throws IndexOutOfBoundsException se a coluna for invalida
     */
    public synchronized String getColumnName(int columnIndex)
	throws IndexOutOfBoundsException
    {
	switch (columnIndex) {
	case  0 : return "File";
	case  1 : return "Path";
	case  2 : return "Description";
	case  3 : return "Status";
	case  4 : return "Filesize (KB)";
	case  5 : return "Transfers Done";
	case  6 : return "Transfers In Progress";
	case  7 : return "Time Left";
	default : throw new IndexOutOfBoundsException();
	}
    }

    /**
     * Devolve o numero de linhas na tabela, que eh o numero de arquivos na
     * fila de push.
     *
     * <p>Origem : extends AbstractTableModel</p>
     *
     * @return numero de linhas da tabela
     */
    public synchronized int getRowCount()
    {
	return this.queue.size();
    }

    /**
     * Retorna o controlador de interface da PushQueue.
     *
     * @return o controlador de interface
     */
    public synchronized UIController getUIController()
    {
	return this.uiController;
    }

    /**
     * Define o controlador de interface da PushQueue.
     *
     * @param uiController controlador
     */
    public synchronized void setUIController(UIController uiController)
    {
	this.uiController = uiController;
    }

    /**
     * Devolve o valor da linha e coluna especificadas. Como a fonte de dados
     * nao eh exatamente uma tabela, mapeia cada um dos campos de um PushItem
     * para uma "coluna virtual".
     *
     * <p>Origem : extends AbstractTableModel</p>
     *
     * @param row linha do dados (item na fila)
     * @param column "coluna virtual" dos dados (campo do item)
     * @return o valor da posicao requisitada
     * @throws IndexOutOfBoundsException quando algum indice for invalido
     */
    public synchronized Object getValueAt(int row, int column)
	throws IndexOutOfBoundsException
    {
	PushItem pi = (PushItem) this.queue.get(row);
	switch (column) {
	case  0 : return pi.getFilename();
	case  1 : return pi.getPath();
	case  2 : return pi.getDescription();
	case  3 : return pi.getStatus();
	case  4 : return new Long(pi.getFilesize() / 1024);
	case  5 : return new Integer(pi.getTransfersDone());
	case  6 : return new Integer(pi.getTransfersInProgress());
	case  7 :
	    long tmax = this
		.uiController
		.getConfiguration()
		.getTreecastTimeout() * 1000;
	    long tmp = pi.getTimeLeft(tmax) / 1000;
	    return new String((tmp / 60) + ":" + (tmp % 60));
	default : throw new IndexOutOfBoundsException();
	}
    }

    /**
     * Adicionar arquivo aa fila de push.
     *
     * @param fn nome do arquivo, com todo o caminho, a ser adicionado
     * @param desc descricao do arquivo
     */
    public synchronized void add(String fn, String desc)
	throws FileNotFoundException, IOException
    {
	Object[] o = this.getUIController().createHeader(fn, desc);
	this.queue.add(new PushItem((Header) o[0], (String) o[1]));
        this.fireTableRowsInserted(this.queue.size()-1, this.queue.size()-1);
    }

    /**
     * Troca as posicoes do elemento indicado e a do elemento seguinte na fila
     * de push. Permanece na mesma posicao caso for o ultimo elemento.
     *
     * @param id identificacao do elemento
     */
    public synchronized void moveDown(int id)
	throws IndexOutOfBoundsException
    {
	PushItem pi1 = (PushItem) queue.get(id);
	PushItem pi2;
	try {
	    pi2 = (PushItem) queue.get(id + 1);
	}
	catch (IndexOutOfBoundsException e) {
	    return;
	}
	queue.set(id, pi2);
	queue.set(id + 1, pi1);
	this.fireTableRowsUpdated(id, id + 1);
    }

    /**
     * Troca as posicoes do elemento indicado e a do elemento anterior na fila
     * de push. Permanece na mesma posicao caso for o primeiro elemento.
     *
     * @param id identificacao do elemento
     */
    public synchronized void moveUp(int id)
	throws IndexOutOfBoundsException
    {
	PushItem pi1;
	try {
	    pi1 = (PushItem) queue.get(id - 1);
	}
	catch (IndexOutOfBoundsException e) {
	    return;
	}
	PushItem pi2 = (PushItem) queue.get(id);
	queue.set(id, pi1);
	queue.set(id - 1, pi2);
	this.fireTableRowsUpdated(id - 1, id);
    }

    /**
     * Processa a queue, parando quem ja terminou, iniciando quem esta na
     * frente e atualizando os dados de todo mundo.
     */
    public synchronized void refresh()
    {
	int j = 0;
	Iterator i = this.queue.iterator();
	long timeout =
	    this.uiController.getConfiguration().getTreecastTimeout() * 1000;
	int maxPushes =
	    this.uiController.getConfiguration()
	    .getUIConfiguration().getPushQueueMaxPushes();
	    
	while (i.hasNext()) {
	    PushItem p = (PushItem) i.next();
	    String s = p.getStatus();
	    if (s.equals("Done")) {
	    }
	    else if (s.equals("Stopping")) {
		if (p.getTransfersInProgress() == 0) {
		    int n = p.getTransfersDone();
		    this.uiController.killTreecast(p.getTreecast());
		    p.buryPush(n);
		    this.fireTableCellUpdated(j, 3);
		}
	    }
	    else if (s.equals("Pushing")) {
		if (p.getTimeLeft(timeout) <= 0) {
		    p.stopPush();
		    this.queueActive--;
		    this.fireTableCellUpdated(j, 3);
		}
	    }
	    else if (s.equals("Queued")) {
		if (this.queueActive < maxPushes) {
		    Treecast t = this.uiController.broadcast(p.getHeader(),
							     p.getPath());
		    p.startPush(t);
		    this.queueActive++;
		    this.fireTableCellUpdated(j, 3);
		}
	    }
	    this.fireTableCellUpdated(j, 5);
	    this.fireTableCellUpdated(j, 6);
	    this.fireTableCellUpdated(j, 7);
	    j++;
	}
    }

    /**
     * Remove elemento da fila de push e devolve elemento removido.
     *
     * @param row linha do elemento a ser removido
     */
    public synchronized PushItem remove(int row)
	throws IndexOutOfBoundsException
    {
	PushItem pi = (PushItem) this.queue.get(row);
	if (pi.getStatus().equals("Stopping")) return null;
	if (pi.getStatus().equals("Pushing")) {
	    pi.stopPush();
	    return null;
	}
	this.queue.remove(row);
	this.fireTableRowsDeleted(row, row);
	return pi;
    }

    /**
     * Remove varios elementos da fila de push e devolve um ArrayList com eles.
     *
     * @param rows linhas dos elementos a serem removidos
     */
    public synchronized ArrayList remove(int[] rows)
	throws IndexOutOfBoundsException
    {
	ArrayList l = new ArrayList();
	for (int i = 0; i < rows.length; i++)
	    l.add(this.remove(rows[i]));
	return l;
    }

    /**
     * Altera as informacoes de um elemento da fila.
     *
     * @param id identificacao do elemento a ser modificado
     * @param fn novo nome do arquivo (com path)
     * @param desc nova descricao
     */
    public synchronized void update(int id, String fn, String desc)
	throws IndexOutOfBoundsException,
	       NullPointerException,
	       SecurityException,
	       FileNotFoundException,
	       IOException
    {
	Object[] o = this.getUIController().createHeader(fn, desc);
	PushItem pi = (PushItem) queue.get(id);
	pi.update((Header) o[0], (String) o[1]);
	queue.set(id, pi);
	this.fireTableRowsUpdated(id, id);
    }
}
