package tcbnet.util;

/**
 * Monitor que responsavel por lidar com Produtores e Consumidores concorrentes
 * de Objetos.
 */
public class ProdConsMonitor {

    private final Object[] array;

    private int putPtr = 0;
    private int takePtr = 0;

    private int emptySlots;
    private int usedSlots = 0;

    private int waitingPuts = 0;
    private int waitingTakes = 0;

    private final Object putCond = new Object ();
    private final Object takeCond = new Object ();

    private static final int CAPACITY = 1024;

    /**
     * Construtor.
     *
     */
    public ProdConsMonitor () {
	this (ProdConsMonitor.CAPACITY);
    }

    /**
     * Construtor.
     *
     * @param capacity O tamanho do buffer de Objetos.
     */
    public ProdConsMonitor (int capacity) {
	if (capacity <= 0) capacity = ProdConsMonitor.CAPACITY;
	this.array = new Object[capacity];
	this.emptySlots = capacity;
    }

    /**
     * Adiciona um Objeto no buffer.
     *
     * @param o Objeto a ser adicionado.
     */
    public void put (Object o) {
	synchronized (this.putCond) {
	    while (this.emptySlots <= 0) {
		this.waitingPuts++;
		try { this.putCond.wait (); }
		catch (InterruptedException ie) {
		    this.putCond.notify ();
		    /** @todo fazer direito */
		    System.out.println("ERRO DE INTERRUPCAO");
		    System.exit(1);
		}
		finally { this.waitingPuts--; }
	    }
	    this.emptySlots--;
	    this.array[this.putPtr] = o;
	    this.putPtr = (this.putPtr + 1) % this.array.length;
	}
	synchronized (this.takeCond) {
	    this.usedSlots++;
	    this.takeCond.notify ();
	}
    }

    /**
     * Adiciona um array de Objeto no buffer.
     *
     * @param obj Um array de objetos a serem adicionados.
     */
    public void putAll (Object[] obj) {
	for (int i = 0; i < obj.length; i++) {
	    this.put (obj[i]);
	}
    }

    /**
     * Retira um Objeto do buffer.
     *
     * @return O Objeto retirado
     */
    public Object take () {
	Object old = null;
	synchronized (this.takeCond) {
	    while (this.usedSlots <= 0) {
		this.waitingTakes++;
		try { this.takeCond.wait (); }
		catch (InterruptedException ie) {
		    this.takeCond.notify ();
		    System.out.println("ERRO DE INTERRUPCAO");
		    System.exit(1);
		}
		finally { this.waitingTakes--; }
	    }
	    this.usedSlots--;
	    old = this.array[this.takePtr];
	    this.array[this.takePtr] = null;
	    this.takePtr = (this.takePtr + 1) % this.array.length;
	}
	synchronized (this.putCond) {
	    this.emptySlots++;
	    this.putCond.notify ();
	}
	return old;
   }

    /**
     * Testa se o buffer esta vazio 
     *
     * @return true se o buffer esta vazio, false caso contrario
     */
    public boolean isEmpty () {
	synchronized (this.takeCond) {
	    return this.usedSlots == 0;
	}
    }
}








